#!/usr/bin/env python3
"""
Raspberry Pi, HQカメラ, RPZ-PIRSを使った
人感センサー検知 & 撮影プログラム
"""

import os
import time
import datetime
import subprocess
from gpiozero import RGBLED, MotionSensor

# オプション
#  HQカメラの場合
#   写真の最大サイズは4056x3040
#   動画(H.264)の最大サイズは1920x1080, 最大画角は1440x1080
image_size = (1440, 1080)  # 撮影サイズの横幅, 高さ
mode = 'photo'  # 写真なら'photo', 動画なら'movie'
pi5 = False  # Raspberry Pi 5の場合True, それ以外はFalse (動画撮影の手順が異なる)
save_dir = '/home/pi'  # 写真や動画を保存するディレクトリ
interval = 60  # 1度撮影した後、再開するまでのインターバル時間[秒]
movie_duration = 10  # 動画の撮影時間[秒]

# RGB LED準備
led = RGBLED(red=18, green=17, blue=22)

# 人感センサー準備
pir = MotionSensor(27)

print('プログラム開始. 人感センサー検知待ち')

while True:
  if pir.value == 1:
    print('人感センサー検知. 撮影開始')

    # LED青
    led.color = (0, 0, 1)

    dt = datetime.datetime.now()

    if mode == 'photo':
      save_file = os.path.join(save_dir, dt.strftime('%y_%m%d_%H%M_%S.jpg'))
      subprocess.run([
          'libcamera-still', '-n', '-o',
          os.path.join(save_file), '-t', '1', '--width',
          str(image_size[0]), '--height',
          str(image_size[1])
      ])

    elif mode == 'movie':
      save_file = os.path.join(save_dir, dt.strftime('%y_%m%d_%H%M_%S.mkv'))
      h264_file = os.path.join(save_dir, 'tmp.h264')
      pts_file = os.path.join(save_dir, 'tmp.txt')
      if pi5:
        subprocess.run([
            'libcamera-vid', '-n', '-o', save_file, '-t',
            str(movie_duration * 1000), '--width',
            str(image_size[0]), '--height',
            str(image_size[1])
        ])
      else:
        subprocess.run([
            'libcamera-vid', '-n', '-o', h264_file, '--save-pts', pts_file, '-t',
            str(movie_duration * 1000), '--width',
            str(image_size[0]), '--height',
            str(image_size[1])
        ])
        subprocess.run(['mkvmerge', '-o', save_file, '--timecodes', '0:' + pts_file, h264_file])
        os.remove(h264_file)
        os.remove(pts_file)
    else:
      raise ValueError('不明なmode指定です')

    print('撮影終了. インターバル時間待機')
    # LED黄
    led.color = (1, 1, 0)

    time.sleep(interval)  # 次の撮影までのインターバル時間
    print('待機終了. 人感センサー検知待ち')

  else:
    # 人感センサー検出されない場合

    # LED緑
    led.color = (0, 1, 0)

    time.sleep(0.01)
